/*
	jstest.js
	Copyright (C) 2005 H. Adam Mill II
	
	Licensed under the Apache License, Version 2.0 (the "License");
	you may not use this file except in compliance with the License.
	You may obtain a copy of the License at
	
		 http://www.apache.org/licenses/LICENSE-2.0
	
	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/

var JsTest = function()
{
	this.verbose 		= false;
	this.count				= 0;
	this.data			 	= [ ]; // { type, msg, threw, value }
};
JsTest.prototype.test = function(fn, unhandled)
{
	if(null == unhandled) unhandled = true;
	try
	{
		fn();
	}
	catch(ex)
	{
		if(unhandled)
			this.data.push( { type : "An unhandled exception occured in the test function", msg : null, threw : ex, value : null } );		
		else
			throw ex;
	}
}
JsTest.prototype.report = function()
{
	if(!this.data.length)
	{
		document.writeln("<h3>All " + this.count + " tests passed.</h3>");
		return;
	}
	document.writeln("<table width=\"100%\" border=\"0\" style=\"border: thin solid black\"><tr bgcolor=\"lightgrey\"><th>Type</th><th>Message</th><th>Threw</th><th>Value</th></tr>");
	var type
	var msg;
	var threw;
	var value;
	for(var loop=0; loop < this.data.length; loop++)
	{
		type =  this.data[loop].type;
		msg = (null == this.data[loop].msg ? "<null>" : String(this.data[loop].msg));
		var threw = this.data[loop].threw;
		if(threw)
		{
			var threwStr = "";
			if(window.Error && threw instanceof Error)
			{
				threwStr = threw.name + "<br />";
				threwStr += threw.message;
			}
			else
				threwStr = String(threw);
		}
		threw =  (null == this.data[loop].threw ? "<null>" : threwStr);
		value =  (null == this.data[loop].value ? "<null>" : String(this.data[loop].value));
		document.writeln("<tr><td>" + type + "</td><td>" + msg + "</td><td>" + threw + "</td><td>" + value + "</td></tr>");
	}
	document.writeln("</table>");
}
JsTest.prototype.assert = function(test, msg)
{
	if("function" == typeof test)
		this.assertFunction(test, msg);
	else if("string" == typeof test)
		this.assertEval(test, msg);
	else
		this.assertBinary(test, msg);
}
JsTest.prototype.assertBinary = function(value, msg)
{	
	this.count++;
	if(!value)
		this.data.push( { type : "boolean assertion failed", msg : msg, threw : null, value : value  } );
}
JsTest.prototype.assertEval = function(value, msg)
{
	this.count++;
	try
	{
		if(!eval(value))
			this.data.push( { type : "evaluation assertion failed", msg : msg, threw : null, value : value } );	
	}
	catch(ex)
	{
		this.data.push( { type : "evaluation assertion failed with exception", msg : msg, threw : ex, value : value } );
	}
}
JsTest.prototype.assertFunction = function(fn, msg)
{
	this.count++;
	try
	{
		if(!fn())
			this.data.push( { type : "function assertion failed", msg : msg, threw : null, value : fn.toString() } );
	}
	catch(ex)
	{
		this.data.push( { type : "function assertion failed with exception", msg : msg, threw : ex, value : null /* fn.toString() */ } );
	}
}
JsTest.prototype.assertThrows = function(fn, msg)
{
	this.count++;
	if(null == msg) msg = "";
	var threw = false;
	try
	{
		fn();
	}
	catch(ex)
	{
		threw = true;
	}
	if(!threw)
		this.data.push( { type : "throw assertion failed", msg : msg, threw : null, value : fn.toString() } );
}
